#!/usr/bin/env python3
#!/nlp/projekty/metatrans2/public_html/python3/bin/python3

import re
import sys

from agenda import Agenda
from chart_parser import ChartParser
from edge import Edge
from log import log


class TopDownChartParser(ChartParser):
    def prepare_agenda(self):
        self.agenda = Agenda(Edge(predicted_by_rule=rule) for rule
                             in self.rules if rule.left['phrase'] == self.root)
        for edge in self.agenda:
            log.debug("agenda       %s", edge)

    def propose_edges(self, agenda_edge):
        # TODO: zrychlení by se dalo dosáhnout na úkor paměti – prostě by si
        # chart ještě pamatoval slovník („index“) levých stran a pod ním
        # seznam – aby se nemusely procházet všechny hrany
        if agenda_edge.closed:
            for chart_edge in self.chart_open_edges:
                joined_edge = self._fundamental_rule(agenda_edge, chart_edge)
                if joined_edge is not None:
                    yield joined_edge, "fundamental  %s" % joined_edge
            return
        else:
            for chart_edge in self.chart_closed_edges:
                joined_edge = self._use_closed_edges_from_chart(agenda_edge,
                                                                chart_edge)
                if joined_edge is not None:
                    yield joined_edge, "closed       %s" % joined_edge

        extended_edge = self._read_terminal(agenda_edge)
        if extended_edge is not None:
            yield extended_edge, "terminal     %s" % extended_edge

        if agenda_edge.end == len(self.tokens):
            # log.debug('             not predicting further from %s',
            #           agenda_edge)
            return

        for predicted_edge in self._predict(agenda_edge):
            if predicted_edge is not None:
                yield predicted_edge, "predict      %s" % predicted_edge

    @staticmethod
    def _fundamental_rule(agenda_edge, chart_edge):
        """
        if E is in the form of [A → α •, j, k]
        then for each edge [B → γ • A β, i, j] in the chart
        create an edge [B → γ A • β, i, k]
        """
        if (agenda_edge.begin == chart_edge.end and
            # TODO: dvě následující podmínky by možná šly sloučit (anebo
            # všechny tři?)
            chart_edge.next_symbol.get('phrase') == agenda_edge.left['phrase']
                # TODO: a co a jak porovnávám tady?
                and agenda_edge.left.match(chart_edge)):
            log.debug('fundamental  %s', agenda_edge)
            log.debug('+            %s', chart_edge)
            fundamental = Edge(extended_from=chart_edge,
                               extended_using=agenda_edge)

            fundamental.copy_attributes(agenda_edge.left)
            return fundamental

    @staticmethod
    def _use_closed_edges_from_chart(agenda_edge, chart_edge):
        """
        if E is in the form of [B → γ • A β, i, j]
        then for each edge [A → α •, j, k] in the chart
        create an edge [B → γ A • β, i, k].
        """
        if (chart_edge.begin == agenda_edge.end and
            agenda_edge.next_symbol.get('phrase') == chart_edge.left['phrase']
                and chart_edge.left.match(agenda_edge)):
            log.debug('closed edge  %s', agenda_edge)
            log.debug('+            %s', chart_edge)
            if chart_edge:
                log.critical('not just ε!')
            closed = Edge(extended_from=agenda_edge, extended_using=chart_edge)

            closed[agenda_edge.position].update(chart_edge.left)
            log.debug('closed added %s', chart_edge.left)

            return closed

    def _read_terminal(self, agenda_edge):
        """
        if E is in the form of [A → α • aj+1 β, i, j]
        create an edge [A → α aj+1 • β, i, j+1].
        """
        position = agenda_edge.begin + agenda_edge.real_offset
        if position >= len(self.tokens):
            return None

        if self._match_terminal(agenda_edge, self.tokens[position]):
            new = Edge(extended_from=agenda_edge, tokens=self.tokens)
            new.copy_attributes()

            return new

    def _match_terminal(self, agenda_edge, token):
        # log.debug("try match    %s = %s", agenda_edge.next_symbol, token)
        # TODO: tady nemůžu pracovat s "?" (optional), to by fakt chtělo
        # ukládat jinak!
        for attr, expected_value in agenda_edge.next_symbol.items():
            if expected_value is None:
                continue
            elif attr not in token:
                if attr != 'phrase':
                    log.debug("no attr      %s has no %s=%s", token, attr,
                              expected_value)
                return False
            elif token[attr] is None:
                # log.debug("null attr    %s (expected: %s)", attr,
                #           expected_value)
                raise AssertionError('Attribute %s is None in %s, not %s' % (
                    attr, token, expected_value))
            try:  # elif isinstance(token[attr], str):
                if not re.fullmatch(expected_value, token[attr]):
                    # log.debug("mismatch     %s=%s (expected: %s)", attr,
                    #           token[attr], expected_value)
                    return False
            except TypeError:  # else:
                for value in token[attr]:
                    if re.fullmatch(expected_value, value):
                        break
                else:
                    log.debug("mismatch     %s=%s (expected: %s)", attr,
                              token[attr], expected_value)
                    return False

        log.debug("match        %s = %s", token, agenda_edge.next_symbol)
        return True

    def _predict(self, agenda_edge):
        """
        if E is in the form of [A → α • B β, i, j]
        then for each grammar rule B → γ ∈ P, create an edge [B → • γ, j, j].
        """
        phrase = agenda_edge.left['phrase']
        left_recursive = (agenda_edge.position == 0 and
                          phrase == agenda_edge[0].get('phrase'))

        for rule in self.rules:
            if left_recursive and rule.left['phrase'] == phrase:
                continue

            # TODO: stačí to? asi jo, na levých stranách nebývá zatím nic…
            # TODO: ale bývá, lepší je použít _match_
            # TODO: (NP c=4) → (k=1 c=4) predikovat nechci
            if rule.left['phrase'] == agenda_edge.next_symbol.get('phrase'):
                predicted = Edge(predicted_from=agenda_edge,
                                 predicted_by_rule=rule)
                # log.warning('predicted    %s from %s', predicted,
                #             agenda_edge.next_symbol)
                yield predicted

#              0–1 (NP g=I n=S c=1) → (word=krásný k=2 g=I n=S c=1) • (NP g=I n=S c=1)
# predict      1–1 (NP c=4) → • (k=1 c=4)       from (NP g=I n=S c=1)

if __name__ == '__main__':
    top_down = TopDownChartParser(argv=sys.argv)
    top_down.parse_from_vertical()
