# věty

from rules import Rules
from .cz_constituents import constituents, optionals
from .cz_verbals import modals, verbals

# TODO: cz_infinite možná nebude potřeba, možná stačí použít (CLAUSE m=F)
#       a nevkládat podmět(nou větu)

# do slovesné fráze (ale ne před ni!) se vloží nepovinné konstituenty
verbals_with_optionals = Rules(verbals.insert_into_all(
    '(OPTIONALS)', inside=True, after=True, repeat_self=False))

# indikativní věty mají na začátku (už povinný) konstituent
indicative_clauses = Rules(verbals_with_optionals.insert_into_all(
    '(CONSTITUENT)', before=True))

# všechny věty mohou mít klitika a interpozice, ale pozor, vedlejší už spíš
# ne před podřadícím výrazem (a ani tečku), takže je nebudu míchat
indicative_clauses_with_clitics = Rules(indicative_clauses.insert_into_all(
    '(CLITICS)', position=1, repeat_self=False))

indicative_clauses_with_interpositions = Rules(
    indicative_clauses_with_clitics.insert_into_all(
        '(INTER)', before=True, after=True, repeat_self=False))

terminated_indicative_clauses = Rules((
    rule + '(OPT_FULLSTOP)'
    for rule in indicative_clauses_with_interpositions))

# přívlastkové vedlejší věty
relatives = [
    '(RELATIVE g n) → (lemma=který|jenž|co g n)',
    '(RELATIVE g n) → (k=7 c) (lemma=který|jenž g n c)',
]

attributive_clauses = Rules(verbals_with_optionals.insert_into_all(
    '(RELATIVE g n)', before=True, update_left='(ATTR_CLAUSE g n)'))
# „kterého bodla včela“ – bez shody s přísudkem
# „který se smál“ – se shodou, ale zatím to asi není potřeba
#                  (interpretace se z toho dostane i tak)

clauses = terminated_indicative_clauses + attributive_clauses

rules = clauses + relatives

# věty:
# samostatné   oznamovací                             CLAUSE (MATRIX_CLAUSE)
#              tázací (s tázacím … na začátku)        INTERROGATIVE_CLAUSE
#              přací (s nějakou částicí na začátku)   ?
# nesamostatné přívlastkové                         ATTR_CLAUSE
#              podmětné/předmětné                   ?
#              adverbiální (časové, místní, způsobové atd.)

rules += Rules((
    #
    # TODO: předmětové vedlejší věty
    #
    # vypadá to, že to není úplně jednoduchý
    #   *že utekli jsme z hodiny
    #   že jsme utekli z hodiny (iniciální, postiniciální)
    #   že ten kluk se ulil z hodiny (preiniciální, iniciální, postiniciální)
    #   že se ulil z hodiny (iniciální, postiniciální)
    #

    # že/co jsem (to) nevěděl
    # že maminka se (asi) ztratila

    # Kdo	kdo	k3yRnSc1,k3yInSc1,k3yQnSc1hP

    '(OBJ_CLAUSE) → (OPT_COMMA) (word="že|co") (CLAUSE) (OPT_COMMA)',

    # kdo to nevěděl
    # komu jsem to nedal
    # na koho naléhal
)).insert_into_all('(OPT_COMMA)', before=True, inside=False, after=True)


# Učím Petra angličtinu (akuzativ životný a neživotný)
# Petr půjčil Pavlovi knihu
# Kočka se dychtivě napila
# Chce/musí to odevzdat (zítra)
# T0D0: dá se udělat m="[^F]" a zároveň kopírovat modus do CLAUSE?
# '(CLAUSE e g n a) → (MODAL m="[^F]") (k=5 e g n a m=F)',
# lepší je parametrizovat až (CLAUSE m=[^F])

kombinace = '''
Chtěli obědvat
Chtěli jsme obědvat
Naši chtěli obědvat
V sobotu chtěli obědvat
Chtěli obědvat až po nás
Naši nechtěli ani obědvat

musím jít domů
domů prý jít musím
'''

_rules = [
    # Bude to odevzdávat až za měsíc.
    '(CLAUSE e g n a m=B) → (k=5 lemma="být" e g n a m=B) (VP m=F a=I)',

    # Bude to muset odevzdat/odevzdávat až za měsíc.
    '(CLAUSE e g n a m=B) → (k=5 lemma="být" e g n a m=B) (VP e g n a m=F)',

    # Umožňuje nám stanovit příznivější ceny pro domácí klienty.
    '(CLAUSE a e g m n) → (k=5 a e g m n) (VP m=F a=P)',

    # Chtěli [jsme tam] jít, ale nakonec [jsme si to] rozmysleli.

    # Chtěl jsem [lyžovat nebo bruslit].
    # (INF) → (INF) (k=8) (INF)
    # (INF) → (k=5 m=F)
    # (INF) → (k=5) (CLITICS) (NP)
    # (CLAUSE) → (MODAL) (CLITICS) (INF)
    # (NP) → (k=3 x=D) (NP)
    # (NP) → (k=1)
]

# primary_positions → secondary_positions
# facultative? (optionals?) (OPTIONALS)? → (NP c=4)
# rules = insert_gradually_into_rules(verbals, '(NP c=4)')

# spony (a jiný odpad):
# – ten generátor bude muset upravovat obě strany pravidla
# – přidá třeba OBJ_ACC a OBJ_ACC2/EXTRA (víc jich nečekám):
#   (VP OBJ_ACC OBJ_ACC2) → (k=5 m="[^F]") (OBJ_ACC)        (OBJ_ACC2)
#                               učím     Petra (+ANIM)   angličtinu (−ANIM)
# – životnost bohužel u ženskýho rodu nevím, to je pěkně blbý
#   musela by se brát z WordNetu, majka to prostě mít nebude

# '(COPULA e g n a m) → (lemma="být" e g n a m)',

# (všichni) jsme hladoví
# '(VP e g n a m) → (lemma="být" e g n a m) (ADJP)',

# TODO: má smysl zpracovávat spony odděleně od ostatních sloves, protože
# ostatní nebudou mít „Petr je učitel“ (nominativ)

# On je pytel blech.
# '(VP e g n a m) → (lemma="být" e g n a m) (NP)',  # je [něco]
